import { useEffect, useState, useRef, MouseEvent } from 'react';

const maxPeakValue = 32768;

type Props = {
  peaks: number[][] | null;
  logicalWidth: number;
  logicalHeight: number;
  strokeStyle: string;
  fillStyle: string;
  style: React.CSSProperties;
};

export const WaveformCanvas: React.FC<Props> = (props: Props) => {
  const canvasRef = useRef<HTMLCanvasElement>(null);

  useEffect(() => {
    const canvas = canvasRef.current;
    if (canvas == null) {
      console.error('no canvas ref available');
      return;
    }

    const ctx = canvas.getContext('2d');
    if (ctx == null) {
      console.error('no 2d context available');
      return;
    }

    ctx.strokeStyle = props.strokeStyle;
    ctx.fillStyle = props.fillStyle;
    ctx.fillRect(0, 0, canvas.width, canvas.height);

    if (props.peaks == null) {
      return;
    }

    const numChannels = props.peaks.length;
    const chanHeight = canvas.height / numChannels;
    for (let i = 0; i < numChannels; i++) {
      const yOffset = chanHeight * i;
      // props.peaks[n].length must equal canvasLogicalWidth:
      for (let j = 0; j < props.peaks[i].length; j++) {
        const val = props.peaks[i][j];
        const height = Math.floor((val / maxPeakValue) * chanHeight);
        const y1 = (chanHeight - height) / 2 + yOffset;
        const y2 = y1 + height;
        ctx.beginPath();
        ctx.moveTo(j, y1);
        ctx.lineTo(j, y2);
        ctx.stroke();
      }
    }
  }, [props.peaks]);

  return (
    <>
      <canvas
        ref={canvasRef}
        width={props.logicalWidth}
        height={props.logicalHeight}
        style={props.style}
      ></canvas>
    </>
  );
};
